package com.yizhi.live.application.util;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.net.URLEncoder;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yizhi.core.application.enums.InternationalEnums;
import com.yizhi.core.application.exception.BizException;
import com.yizhi.live.application.constant.*;
import com.yizhi.live.application.domain.LiveReplay;
import com.yizhi.live.application.enums.LiveStatusV2;
import com.yizhi.live.application.enums.PolyvReqEnum;
import com.yizhi.live.application.vo.LiveStatisticDataVo;
import com.yizhi.live.application.vo.PolyvResponseVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMethod;

import java.nio.charset.Charset;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Date 2020/12/8 2:51 下午
 * @Author lvjianhui
 **/
public class PolyvUtils {

    public static Logger logger = LoggerFactory.getLogger(PolyvUtils.class);

    public String getRedirectUrl(String active) {
        if ("dev".equals(active)) {
            return UtilConstants.DEV_LIVE_CUSTOM_URI;
        } else if ("sit".equals(active)) {
            return UtilConstants.SIT_LIVE_CUSTOM_URI;
        } else if ("uat".equals(active)) {
            return UtilConstants.UAT_LIVE_CUSTOM_URI;
        } else {
            return UtilConstants.PRO_LIVE_CUSTOM_URI;
        }
    }

    /**
     * 直播状态
     * 按照时间维度（当前时间与当前直播设置的时间对比）
     * 1、当前时间<开始时间
     * 正在直播：进行中
     * 没在直播：未开始
     * 2、开始时间<当前时间<结束时间
     * 正在直播：进行中
     * 没在直播∩[开始时间到当前时间]没直播过：未开始
     * 没在直播∩[开始时间到当前时间]直播过：直播回放
     * 3、当前时间>结束时间
     * 正在直播：进行中
     * 没在直播∩[开始时间到当前时间]没直播过：已结束
     * 没在直播∩[开始时间到当前时间]直播过：直播回放
     *
     * @param channelId
     * @param startAt
     * @param endAt
     * @param liveStatus see LiveStatusEnum
     * @return
     */
    public static LiveStatusV2 getLiveStatus(String channelId, Date startAt, Date endAt, Integer liveStatus) {
        // 直播中
        if (null != liveStatus && liveStatus.equals(LiveStatusEnum.LIVE.getCode())) {
            logger.info("直播 channel:" + channelId+ "-----1");
            return LiveStatusV2.RUNNING;
        }

        if (LiveStatusV2.RUNNING.getCode().equals(getThirdLiveStatus(channelId))) {
            logger.info("直播 channel:" + channelId+ "-----2");
            return LiveStatusV2.RUNNING;
        }

        Long currentTime = System.currentTimeMillis();

        // 未开始
        if (startAt.getTime() > currentTime) {
            logger.info("直播 channel:" + channelId+ "-----3");
            return LiveStatusV2.UN_START;
        }
        Integer count = getCountByChannel(channelId, startAt, new Date(currentTime));
        if (count > 0) {
            logger.info("直播 channel:" + channelId+ "-----4");
            return LiveStatusV2.REPLAY;
        }
        if (count <= 0 && currentTime > endAt.getTime()) {
            logger.info("直播 channel:" + channelId+ "-----5");
            return LiveStatusV2.END;
        }
        logger.info("直播 channel:" + channelId+ "-----6");
        return LiveStatusV2.UN_START;
    }

    public static Integer getThirdLiveStatus(String channelId) {
        Map<String, Object> params = new HashMap();
        params.put("appId", UtilConstants.APP_ID);
        params.put("timestamp", System.currentTimeMillis() + "");
        params.put("channelIds", channelId);
        try {
            PolyvResponseVo<JSONArray> result = execPolyvApi(null, params, UtilConstants.LIVE_STATUS_URL, null);
            JSONArray json = result.getData();
            if (json == null || json.size() == 0) {
                return LiveStatusV2.END.getCode();
            }
            if (JSONObject.toJSONString(json.get(0)).contains("live")) {
                return LiveStatusV2.RUNNING.getCode();
            }
            return LiveStatusV2.END.getCode();
        } catch (Exception e) {
            logger.error("获取直播状态出错 channelId:{}", channelId, e);
            return LiveStatusV2.END.getCode();
        }
    }

    /**
     * https://api.polyv.net/live/v3/channel/session/list
     * 查询频道直播场次信息
     *
     * @param channelId
     * @return
     */
    public static Integer getCountByChannel(String channelId, Date startAt, Date endAt) {
        String url = "https://api.polyv.net/live/v3/channel/session/list";
        Long timeStamp = System.currentTimeMillis();
        Map<String, Object> signParams = new HashMap<String, Object>();
        signParams.put("appId", UtilConstants.APP_ID);
        signParams.put("channelId", channelId);
        signParams.put("timestamp", timeStamp);
        signParams.put("endDate", DateUtil.formatDate(endAt));
        signParams.put("startDate", DateUtil.formatDate(startAt));
        try {
            PolyvResponseVo<JSONObject> vo = execPolyvApi(null, signParams, url, RequestMethod.GET);
            Integer count = vo.getData().getInteger("totalItems");
            if (count > 0) {
                String contents = vo.getData().getString("contents");
                List<PolyvLiveCount> list = JSONObject.parseArray(contents, PolyvLiveCount.class);
                for (PolyvLiveCount polyvLiveCount : list) {
                    if (polyvLiveCount.getStartTime() >= startAt.getTime() &&
                            polyvLiveCount.getEndTime() <= endAt.getTime()) {
                        return count;
                    }
                }
                return 0;
            }
        } catch (Exception e) {
            logger.warn("获取直播场次出错 channelId:{}", channelId, e);
        }
        return 0;
    }


    /**
     * 保利威直播参数修改
     * http://api.polyv.net/live/v3/channel/basic/update
     * http://api.polyv.net/live/v3/channel/basic/update?appId={{appId}}&timestamp={{timestamp}}&channelId={{channelId}}&sign={{sign}}
     *
     * @param needAuthView 授权观看时传 trye,esle false
     * @see "https://dev.polyv.net/2019/liveproduct/l-api/zbglgn/pdcz/update-channel-detail-setting/"
     */
    public static PolyvResponseVo<String> updateChannelParams(String channelId, PolyvUpdatePO params, Boolean needAuthView, String customKey, String customUri) {
        Long timeStamp = System.currentTimeMillis();
        Map<String, Object> signParams = new HashMap<String, Object>();
        signParams.put("appId", UtilConstants.APP_ID);
        signParams.put("channelId", channelId);
        signParams.put("timestamp", timeStamp);

        JSONObject json = new JSONObject();

        JSONArray array = new JSONArray();
        JSONObject authSettings = new JSONObject();
        String enabled = "N";
        if (needAuthView) {
            enabled = "Y";
        }
        authSettings.put("enabled", enabled);
        authSettings.put("rank", 1);
        authSettings.put("authType", "custom");
        authSettings.put("customUri", URLEncoder.createDefault().encode(customUri, Charset.defaultCharset()));
        authSettings.put("customKey", customKey);
        array.add(authSettings);

        json.put("basicSetting", params);
        json.put("authSettings", array);
        String url = String.format(UtilConstants.CHANNEL_ALL_INFO_UPDATE, UtilConstants.APP_ID, timeStamp, channelId, getSign(signParams));
        return execPolyvApi(json.toJSONString(), null, url, RequestMethod.POST);
    }


    /**
     * 查询多个频道的时间段内的统计数据
     *
     * @param channelIds 多个频道
     * @param startDay   开始日期 yyyy-mm-dd
     * @param endDay     截至日期 yyyy-mm-dd
     */
    public static List<LiveStatisticDataVo> getLiveStatistic(String channelIds, Date startDay, Date endDay) {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        if (null == startDay || null == endDay || StringUtils.isEmpty(channelIds)) {
            return new ArrayList<>();
        }
        String startDate = format.format(startDay);
        String endDate = format.format(endDay);
        String url = String.format(UtilConstants.LIVE_CHANNELS_STATISTIC_DATA, UtilConstants.APP_USER_ID);
        Map<String, Object> params = new HashMap<>();
        params.put(PolyvReqEnum.APP_ID.getName(), UtilConstants.APP_ID);
        params.put(PolyvReqEnum.TIMESTAMP.getName(), System.currentTimeMillis() + "");
        params.put(PolyvReqEnum.CHANNEL_IDS.getName(), channelIds);
        params.put(PolyvReqEnum.START_DATE.getName(), startDate);
        params.put(PolyvReqEnum.END_DATE.getName(), endDate);
        PolyvResponseVo<JSONArray> result = execPolyvApi(null, params, url, RequestMethod.GET);
        JSONArray data = result.getData();
        return JSONObject.parseArray(JSONObject.toJSONString(data), LiveStatisticDataVo.class);

    }

    public static List<LiveReplay> getChannelReplay(String channelId) {
        String url = String.format(UtilConstants.LIVE_REPLAY_LIST, channelId);
        Map<String, Object> params = new HashMap<>();
        params.put("appId", UtilConstants.APP_ID);
        params.put("timestamp", System.currentTimeMillis() + "");
        params.put("page", "1");
        params.put("pageSize", "100");
        params.put("listType", "playback");
        PolyvResponseVo<JSONObject> reslut = execPolyvApi(null, params, url, RequestMethod.GET);
        JSONArray listObject = (JSONArray) reslut.getData().get("contents");
        List<LiveReplay> list = JSONObject.parseArray(JSONObject.toJSONString(listObject), LiveReplay.class);
        if (list == null) {
            list = new ArrayList<LiveReplay>();
        }
        return list;
    }

    public static <T> PolyvResponseVo<T> execPolyvApi(String requestBody, Map<String, Object> params, String apiUrl, RequestMethod method) {
        String result;
        if (method == null || method == RequestMethod.GET) {
            String sign = getSign(params);
            params.put("sign", sign);
            result = HttpUtil.get(apiUrl, params);
        } else {
            result = HttpUtil.post(apiUrl, requestBody);
        }

        logger.info("调用保利威 URL:{} params:{} requestBody:{} result:{}", apiUrl, params, requestBody, result);
        try {
            PolyvResponseVo<T> vo = JSONObject.parseObject(result, PolyvResponseVo.class);
            if (vo.getCode() != 200) {
                logger.error("调用保利威 ERROR MSG:{}", vo.getMessage());
                throw new BizException(InternationalEnums.DOCUMENTRELATIONCONTROLLER6);
            }
            return vo;
        } catch (Exception e) {
            logger.error("调用保利威 ERROR MSG:" + result, e);
            throw new BizException(InternationalEnums.DOCUMENTRELATIONCONTROLLER6);
        }
    }

    public static String getSign(Map<String, Object> paramMap) {
        String appSecret = UtilConstants.APP_SECRET;
        //对参数名进行字典排序
        String[] keyArray = paramMap.keySet().toArray(new String[0]);
        Arrays.sort(keyArray);
        //拼接有序的参数串
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append(appSecret);
        for (String key : keyArray) {
            stringBuilder.append(key).append(paramMap.get(key));
        }

        stringBuilder.append(appSecret);
        String signSource = stringBuilder.toString();

        String sign = org.apache.commons.codec.digest.DigestUtils.md5Hex(signSource).toUpperCase();
        return sign;
    }

}
